@@ -313,6 +313,17 @@ class TestAssertionRewrite:
             assert "%test" == "test"
         assert getmsg(f).startswith("assert '%test' == 'test'")
 
+    def test_custom_repr(self):
+        def f():
+            class Foo(object):
+                a = 1
+
+                def __repr__(self):
+                    return "\n{ \n~ \n}"
+            f = Foo()
+            assert 0 == f.a
+        assert r"where 1 = \n{ \n~ \n}.a" in util._format_lines([getmsg(f)])[0]
+
 
 class TestRewriteOnImport:
 
@@ -500,13 +511,13 @@ class TestAssertionRewriteHookDetails(object):
         state = AssertionState(config, "rewrite")
         source_path = tmpdir.ensure("source.py")
         pycpath = tmpdir.join("pyc").strpath
-        assert _write_pyc(state, [1], source_path, pycpath)
+        assert _write_pyc(state, [1], source_path.stat(), pycpath)
         def open(*args):
             e = IOError()
             e.errno = 10
             raise e
         monkeypatch.setattr(b, "open", open)
-        assert not _write_pyc(state, [1], source_path, pycpath)
+        assert not _write_pyc(state, [1], source_path.stat(), pycpath)
 
     def test_resources_provider_for_loader(self, testdir):
         """
